const BACKGROUND_COLOR = "#333",
  DEFAULT_GAZE_POINT_RADIUS = 15,
  DEFAULT_GAZE_POINT_COLOR = "#ff007b",
  FPS_FONT = "16px ShareTech",
  FPS_COLOR = "#ffffff",
  MAX_GAZE_POINT_AGE = 500;

var lastUpdate,
  lastDelta,
  fpsBuffer = [];

class GameManger {

  constructor() {
    this.gazePoints = new Map();
  }

  setCanvas(canvas) {
    this.canvas = canvas;
    this.context = this.canvas.getContext("2d", { alpha: false });
    this.canvas.style.backgroundColor = BACKGROUND_COLOR;
  }

  setFrameRate(fps) {
    this.fps = fps;
  }

  showFPS() {
    this.shouldDisplayFrameRate = true;
  }

  hideFPS() {
    this.shouldDisplayFrameRate = false;
  }

  setSize(width, height) {
    this.canvas.width = width;
    this.canvas.height = height;
    this.canvas.style.width = `${width}px`;
    this.canvas.style.height = `${height}px`;
  }

  start() {
    let targetFrameTime = 1000 / this.fps;
    this.setUpdateTime();
    this.loop = setInterval(this.onTick.bind(this), targetFrameTime);
  }

  setUpdateTime() {
    lastUpdate = Date.now();
  }

  setDelta() {
    lastDelta = Date.now() - lastUpdate;
  }

  addGazePoint(point) {
    this.gazePoints.set(point.id, point);
  }

  removeGazePoint(point) {
    this.gazePoints.delete(point.id);
  }

  onTick() {
    this.setDelta();
    this.updateGazePoints();
    this.context.clearRect(0, 0, this.canvas.width, this.canvas.height);
    this.drawGazePoints();
    if (this.shouldDisplayFrameRate === true) {
      this.drawFrameRate();
    }
    this.setUpdateTime();
  }

  updateGazePoints() {
    let now = Date.now();
    for (let item of this.gazePoints) {
      let point = item[1];
      if (now - point.createdAt > MAX_GAZE_POINT_AGE) {
        this.removeGazePoint(point);
      }
    }
  }

  drawGazePoints() {
    let now = Date.now();
    this.context.fillStyle = DEFAULT_GAZE_POINT_COLOR;
    for (let item of this.gazePoints) {
      let relativeAge = 1 - ((now - item[1].createdAt) /
        MAX_GAZE_POINT_AGE);
      this.context.save();
      this.context.globalAlpha = relativeAge;
      this.context.beginPath();
      this.context.ellipse(item[1].targetX, item[1].targetY,
        relativeAge * DEFAULT_GAZE_POINT_RADIUS, relativeAge * DEFAULT_GAZE_POINT_RADIUS, Math.PI / 4,
        0, 2 * Math.PI);
      this.context.fill();
      this.context.closePath();
      this.context.restore();
    }
  }

  drawFrameRate() {
    let fps = parseInt(1000 / lastDelta),
    averageFps;
    fpsBuffer.push(fps);
    if(fpsBuffer.length === this.fps) {
      averageFps = parseInt(fpsBuffer.reduce((a,b) => a + b, 0) / fpsBuffer.length);
      fpsBuffer.shift();
    }
    this.context.beginPath();
    this.context.font = FPS_FONT;
    this.context.fillStyle = FPS_COLOR;
    this.context.fillText(`FPS: ${averageFps}`, 30, 30);
    this.context.closePath();
  }

}

export default GameManger;